@@ -101,7 +101,8 @@ def write_workbook(
         sheets,
         theme = None,
         cover = None,
-        contentsheet = "Contents",
+        contentsheet = None,
+        contentsheet_label = "Contents",
         contentsheet_options = {},
         notes_table = None,
         notesheet_label = "Notes",
@@ -116,6 +117,9 @@ def write_workbook(
     information. It calls upon the package to write a formatted `.xlsx`
     file to the specified path.
 
+    .. note:: Deprecated in v1.1.0: `contentsheet` will be removed
+        in v2, it is replaced by `contentsheet_label`
+
     Parameters
     ----------
     filename : str
@@ -145,17 +149,22 @@ def write_workbook(
     auto_width : bool, optional
         indicate if column widths should be automatically determined. True by
         default.
+    contentsheet : str
+        alias for contentsheet_label, deprecated in v1.1.0
 
     Returns
     -------
     None
     """
+    if contentsheet is not None:
+        contentsheet_label = contentsheet
+
     wb = produce_workbook(
         filename,
         sheets,
         theme,
         cover,
-        contentsheet,
+        contentsheet_label,
         contentsheet_options,
         notes_table,
         notesheet_label,
